@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -28,14 +28,14 @@
 #if defined(i386) || defined(__i386)
 #if !HAVE_PAD128_T
 typedef union {
-    long double	_q;
-    int32_t		_l[4];
+    long double _q;
+    int32_t     _l[4];
 } pad128_t;
 #endif
 #if !HAVE_UPAD128_T
 typedef union {
-    long double	_q;
-    uint32_t	_l[4];
+    long double _q;
+    uint32_t    _l[4];
 } upad128_t;
 #endif
 #endif
@@ -48,6 +48,11 @@ typedef union {
 #include <sys/resource.h>
 SQUIDCEXTERN int getrusage(int, struct rusage *);
 
+// Solaris 11 needs this before <sys/socket.h> to get the definition for msg_control
+// and possibly other type definitions we dont know about specifically
+#define _XPG4_2 1
+#include <sys/socket.h>
+
 /**
  * prototypes for system function missing from system includes
  * on some Solaris systems.
@@ -58,13 +63,6 @@ SQUIDCEXTERN int getpagesize(void);
 SQUIDCEXTERN int gethostname(char *, int);
 #endif
 
-/*
- * SunPro CC handles extern inline as inline, PLUS extern symbols.
- */
-#if !defined(_SQUID_EXTERNNEW_) && defined(__SUNPRO_CC)
-#define _SQUID_EXTERNNEW_ extern
-#endif
-
 /*
  * SunStudio CC does not define C++ portability API __FUNCTION__
  */
@@ -105,3 +103,4 @@ SQUIDCEXTERN int gethostname(char *, int);
 
 #endif /* _SQUID_SOLARIS_ */
 #endif /* SQUID_OS_SOALRIS_H */
+
