@@ -1,6 +1,16 @@
+/*
+ * Copyright (C) 1996-2016 The Squid Software Foundation and contributors
+ *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
+ */
+
 #include "squid.h"
+#include "cache_cf.h"
+#include "ConfigParser.h"
+#include "Debug.h"
 #include "format/Config.h"
-#include "protos.h"
 #include <list>
 
 Format::FmtConfig Format::TheConfig;
@@ -10,10 +20,12 @@ Format::FmtConfig::parseFormats()
 {
     char *name, *def;
 
-    if ((name = strtok(NULL, w_space)) == NULL)
+    if ((name = ConfigParser::NextToken()) == nullptr) {
         self_destruct();
+        return;
+    }
 
-    if ((def = strtok(NULL, "\r\n")) == NULL) {
+    if ((def = ConfigParser::NextQuotedOrToEol()) == nullptr) {
         self_destruct();
         return;
     }
@@ -39,5 +51,6 @@ Format::FmtConfig::registerTokens(const String &nsName, TokenTableEntry const *t
     if (tokenArray != NULL)
         tokens.push_back(TokenNamespace(nsName, tokenArray));
     else
-        debugs(0,0, "BUG: format tokens for '" << nsName << "' missing!");
+        debugs(0, DBG_CRITICAL, "BUG: format tokens for '" << nsName << "' missing!");
 }
+
